// Here goes prototypes declaration and documentation

#include <windows.h>
#include "define.h" /* this include file stores typedef that can be easily used */

#ifdef __cplusplus 
    extern "C" {
#endif

/**
* \mainpage
* This board has two main functionalities : \n
* <ul>
* <li>4 digital inputs and 3 digital outputs</li>
* <li>4 modules with 4 analog inputs per module</li>
* </ul>
* \section digital_section Digital inputs / outputs
* You have the possibility to read the status of the digital inputs (one bit per channel - 4 channels). \n
* You also have the possibility to set the statos of the digital outputs (again, one bit per channel - 3 channels). \n
* The digital inputs are not interruptible. See the module xPCI-3200 digital inputs/outputs functions for more information about the functions. \n
* You can also find examples for C and C#.
* \section analog_section Analog inputs
* On the board, there is 4 analog input modules, that enables to acquire temperature. \n
* If you use RTD sensors, you can acquire up to 2 sensors per module (so 8 sensors). \n
* By using TC sensors, in single-ended module you can acquire up to 4 sensors per module (so 16 sensors). \n
* To get the temperature of a sensor, you have two different possibilities: reading one sensor or the scan mode. \n
* \subsection onesensor_section Reading one sensor
* In the module xPCI-3200 Temperature function you will have documentation about the function that enables to read one sensor. \n
* Pros: You can choose the gain of the acquisition (more precise). \n
* Cons: You cannot reach high frequencies.
* \subsection scan_section Scan
* The scan method enables to acquire all the sensors of the module in a continuous way, with a defined frequency. Have a look on the xPCI-3200 Scan functions module to see the available functions. \n
* Pros: You can define a constant frequency, and then receive data regulary.
* Cons: You cannot set the gain, it is set to x1.
**/

/** \defgroup PCI3200 xPCI-3200 functions
 * @{
 *        \addtogroup General xPCI-3200 general functions
 *        \ingroup PCI3200
 *        @{ 
 */
 
/**  Enables to get the number of xPCI-3200 boards present in the system. \n
 * The function fill the value pointed by pb_NbrOfBoards with the number of boards present.
 *                                                                            
 * @param[out] pb_NbrOfBoards Value that will be filled with the number of boards     
 * @retval    0 : No error
 * @retval -1 : Error while getting the number of boards
 */
_INT_ i_PCI3200_GetNumberOfBoards (PBYTE    pb_NbrOfBoards);

/** Enables to get the number of modules of the board \n
 * The function fills the value pointed by pdw_NumberOfModules with the number of modules present.
 *
 * @param[in] h_DeviceHandle       Handle of the board
 * @param[out] pdw_NumberOfModules Number of modules of the board
 * @retval 0 : No error
 * @retval -1 : Error while getting the number of modules
 */
_INT_ i_PCI3200_GetNumberOfModules(HANDLE h_DeviceHandle, PDWORD pdw_NumberOfModules);

/** Get device information of the board.
 *                                                                   
 * @param[in] b_BoardIndex : Index of the board
 * @param[in] dw_IdentifierStringSize : size of the identifier parameter 
 * @param[out] pc_Identifier: Identifier of the board
 * @param[out] pdw_UINumber : This number is typically a user-perceived slot number, such as a number printed next to the slot on the board, or some other number that makes locating the physical device easier for the user. If the device is on a bus that has no UI number convention, or if the bus driver for the device cannot determine the UI number, this value is 0xFFFFFFFF.
 * @param[out] pdw_DeviceNumber : Device number.
 * @param[out] pdw_BusNumber : Bus number.
 * @param[out] pdw_BaseAddress0 : Base address 0 of the board
 * @param[out] pdw_BaseAddress1 : Base address 1 of the board
 * @param[out] pdw_BaseAddress2 : Base address 2 of the board
 * @param[out] pdw_BaseAddress3 : Base address 3 of the board
 * @param[out] pb_Interrupt : Interrupt line of the board
 * @retval     0 : No error
 * @retval -1 : Unable to open the board via index
 * @retval -2 : Error when calling driver's IOCTL
 */
_INT_ i_PCI3200_GetBoardInformation(BYTE        b_BoardIndex,
                                    DWORD        dw_IdentifierStringSize,
                                    PCHAR        pc_Identifier,
                                    PDWORD        pdw_UINumber,
                                    PDWORD        pdw_DeviceNumber,
                                    PDWORD        pdw_BusNumber,
                                    PDWORD        pdw_BaseAddress0,
                                    PDWORD        pdw_BaseAddress1,
                                    PDWORD        pdw_BaseAddress2,
                                    PDWORD        pdw_BaseAddress3,
                                    PBYTE        pb_Interrupt);
                                   
/** Open the board with the index : b_BoardIndex. A handle is returned to the \n    
 * user which allows to use the driver functions. \n 
 *                                                                                  
 * @param[in] b_BoardIndex : Index of the board to open                             
 * @param[out] ph_DeviceHandle : Handle of the selected xPCI-3200 board for using the functions
 * @retval 0 : No error
 * @retval -1 : Unable to get the number of available boards
 * @retval -2 : Wrong device index parameter. The parameter is too high
 * @retval -3 : Error while getting the device's class
 * @retval -4 : Error while getting informations about the device
 * @retval -5 : Memory allocation error
 * @retval -6 : Get device interface detail error
 * @retval -7 : Error while creating communication handle
 * @retval -8 : Error in the initialization callback function
 */
_INT_ i_PCI3200_OpenBoardViaIndex (BYTE    b_BoardIndex,
                                   PHANDLE    ph_DeviceHandle);    

/** Open the board with the indentifier : pc_Identifier. A handle is returned to the \n    
 * user which allows to use the following functions. \n                                                         
 *                                                                                  
 * @param[in] pc_Identifier : Identifier of the board to open
 * @param[out] ph_DeviceHandle : Handle of the board xPCI-3200 for using the functions
 * @retval 0 : No Error
 * @retval -1 : Unable to get the number of available boards
 * @retval -2 : Unable to get the board identifier
 * @retval -3 : Unable to open the board
 * @retval -4 : Identifier not found
 */
_INT_ i_PCI3200_OpenBoardViaIdentifier (PCHAR    pc_Identifier,
                                        PHANDLE    ph_DeviceHandle);
                                        
/** Release the handle of the board. Block the access to the board.
 *                                                                   
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 */
_INT_ i_PCI3200_CloseBoard (HANDLE    h_DeviceHandle);

/** Return the version of the DLL.
 *                                                                   
 * @param[out] pdw_Version  : Version. sample : 0x01010910 -> version 1.01 september 2010       
 * @retval 0 : No Error
 */
_INT_ i_PCI3200_GetDLLVersion (PDWORD    pdw_Version);

/*Function's Name   : i_PCI3200_GetDeviceDriverVersion*/
/** Return the version of the device driver.
*                                                                   
* @param[in] h_DeviceHandle  : Handle of the board                
* @param[out] pdw_Version  : Version. sample : 0x01011211 -> version 1.01 - 12/2011
* @retval 0 : No Error
* @retval -1 : Error while calling IOCTL function
*/
_INT_ i_PCI3200_GetDeviceDriverVersion(HANDLE h_DeviceHandle, PDWORD pdw_Version);

/**
 *        @}
 *        
 *        \addtogroup Interrupt xPCI-3200 interrupt management functions
 *         \ingroup PCI3200
 *        @{ 
 */
 
/** Install an interrupt callback function.
 * This function will be called each time the board generate an interrupt. \n
 *
 * @param[in] h_DeviceHandle        : Handle of the board                
 * @param[in] v_FunctionName        : Address of the callback function
 *        \n    \b h_DeviceHandle        : Handle of the board
 *        \n    \b b_FIFOOverflow        : If uc_FIFOOverflow != 0, there are too many untreated interrupts in the kernel FIFO, and any new information is lost
 *        \n    \b dw_InterruptSource    : Source of the interrupt. Contains the module which generated the interrupt (0,1,2 or 3)
 *        \n    \b dw_InterruptMask        : Ground of the interrupt (0x1 means the scan conversion is available)
 *        \n    \b pd_Args                : Values read during the interrupt. If the dw_InterruptMask is 0x1, the array contains the measured temperature (in Celcius)
 *        \n    \b dw_ArgsCount            : Number of values read during the interrupt
 *
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : An interrupt routine was still installed. Remove it with i_PCI3200_ResetBoardIntRoutine and retry
 * @retval -3 : Error while creating kill event
 * @retval -4 : Error while creating interrupt event
 * @retval -5 : Error while creating interrupt thread
 * @retval -6 : Error while changing the priority class of the thread
 * @retval -7 : Error while changing the priority of the thread
 * @retval -8 : Error while calling the IOCTL command
 * @retval -9 : Error in the interrupt callback function
 */
_INT_ i_PCI3200_SetBoardIntRoutine (HANDLE    h_DeviceHandle,
                                    void    (*v_FunctionName)(   HANDLE    h_DeviceHandle,
                                                                BYTE    b_FIFOOverflow,
                                                                DWORD    dw_InterruptSource,
                                                                DWORD    dw_InterruptMask,
                                                                DOUBLE * pd_Args,
                                                                DWORD    dw_ArgsCount)
);

/** Deinstall the interrupt callback function.
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : No interrupt routine installed
 * @retval -3 : Error in the reset interrupt callback function
 */
_INT_ i_PCI3200_ResetBoardIntRoutine (HANDLE    h_DeviceHandle);

/**
 *        @}
 *        
 *        \addtogroup Digital xPCI-3200 digital inputs/outputs functions
 *         \ingroup PCI3200
 *        @{ 
 */

/** Read 1 digital input of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to read (0 to 3)
 * @param[out] pb_Value Status of the digital input
 *
 * @retval 0 : No Error
 * @retval -1 : Invalid channel
 * @retval -2 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Read1DigitalInput(HANDLE    h_DeviceHandle,
                                  BYTE        b_Channel,
                                  PBYTE        pb_Value);

/** Read the 4 digital inputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_Value Status of the digital inputs
 *
 * @retval 0 : No Error
 * @retval -1 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Read4DigitalInputs(HANDLE    h_DeviceHandle,
                                   PBYTE    pb_Value);

/** Set 1 digital output of the board to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to set (0 to 2)
 *
 * @retval 0 : No Error
 * @retval -1 : Wrong channel
 * @retval -3 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Set1DigitalOutputOn(HANDLE    h_DeviceHandle,
                                  BYTE        b_Channel);

/** Set 1 digital output of the board to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to reset (0 to 2)
 *
 * @retval 0 : No Error
 * @retval -1 : Wrong channel
 * @retval -3 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Set1DigitalOutputOff(HANDLE    h_DeviceHandle,
                                     BYTE    b_Channel);

/** Set the digital outputs set in the mask to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to set (0x0 to 0x7)
 *
 * @retval 0 : No Error
 * @retval -1 : Wrong mask
 * @retval -3 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Set3DigitalOutputsOn(HANDLE    h_DeviceHandle,
                                     BYTE    b_Mask);

/** Set the digital outputs set in the mask to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to reset (0x0 to 0x7)
 *
 * @retval 0 : No Error
 * @retval -1 : Wrong mask
 * @retval -3 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Set3DigitalOutputsOff(HANDLE    h_DeviceHandle,
                                      BYTE    b_Mask);

/** Enable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until calling i_PCI3200_Set1DigitalOutputOff or i_PCI3200_Set4DigitalOutputsOff. \n
 * When digital output memory is disabled, the active channels are reset at each call of i_PCI3200_Set1DigitalOutputOn or i_PCI3200_Set4DigitalOutputsOn.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @retval 0 : No Error
 * @retval -1 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_SetDigitalOutputMemoryOn(HANDLE h_DeviceHandle);

/** Disable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until calling i_PCI3200_Set1DigitalOutputOff or i_PCI3200_Set4DigitalOutputsOff. \n
 * When digital output memory is disabled, the active channels are reset at each call of i_PCI3200_Set1DigitalOutputOn or i_PCI3200_Set4DigitalOutputsOn.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @retval 0 : No Error
 * @retval -1 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_SetDigitalOutputMemoryOff(HANDLE h_DeviceHandle);

/** Get the status of 1 digital output of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Channel Channel to get (0 to 3)
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @retval 0 : No Error
 * @retval -1 : Wrong channel
 * @retval -2 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Get1DigitalOutputStatus(HANDLE    h_DeviceHandle,
                                        BYTE    b_Channel,
                                        PBYTE    pb_OutputsStatus);

/** Get the status of all the digital outputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @retval 0 : No Error
 * @retval -1 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_Get3DigitalOutputsStatus(HANDLE    h_DeviceHandle,
                                         PBYTE    pb_OutputsStatus);

/**
 *        @}
 *        
 *        \addtogroup Scan xPCI-3200 Scan functions
 *         \ingroup PCI3200
 *        @{ 
 */

/**
* Initialize the sensor connected to a analog-input module of the board.
* @param[in] h_DeviceHandle        Handle of the board
* @param[in] b_ModuleIndex        Index of the module (0-3)
* @param[in] b_SensorType        The type of the sensor connected (ADDIDATA_THERMOCOUPLE for TC sensors, ADDIDATA_RTD for RTD sensors)
* @param[in] b_SensorModel        A array that contains the model of the sensors that are connected. \n
* If b_SensorType is ADDIDATA_THERMOCOUPLE : \n
* <ul>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_B</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_E</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_J</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_K</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_N</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_R</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_S</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_T</li>
* </ul>
* If b_SensorType is ADDIDATA_RTD : \n
* <ul>
* <li>ADDIDATA_RTD_TYPE_PT</li>
* <li>ADDIDATA_RTD_TYPE_Ni</li>
* </ul>
* @param[in] b_ConnectionType    The type of the connection that is used (APCI3200_CONNECTION_TYPE_SINGLE for single-ended, and APCI3200_CONNECTION_TYPE_DIFFERENTIAL for differential) \n
* <b> If b_SensorType is RTD, b_ConnectionType must be set to APCI3200_CONNECTION_TYPE_DIFFERENTIAL </b>
* @param[in] d_RtdResistance    A array that contains the resistance R0 of the RTD sensors. Only fill this array if you use RTD sensors. \n
* R0 is the resistance of your sensor for a temperature of 0 Celcius
* @retval 0 :     No error
* @retval -1 :    The h_DeviceHandle parameter is wrong
* @retval -2 :    The b_ModuleIndex parameter is wrong
* @retval -3 :    The b_SensorType parameter is wrong
* @retval -4 :    The b_ConnectionType parameter is wrong
* @retval -5 :    One of the sensor model in b_SensorModel is wrong (not in the TC sensors model list)
* @retval -6 :    b_SensorType is set to ADDIDATA_RTD, but the connection type is not set to APCI3200_CONNECTION_TYPE_DIFFERENTIAL
* @retval -7 :    One of the sensor model in b_SensorModel is wrong (not in the RTD sensors model list)
* @retval -8 :    Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCI3200_InitScanModuleAcquisitionSensor(HANDLE    h_DeviceHandle,
                                                BYTE b_ModuleIndex,
                                                BYTE b_SensorType,
                                                BYTE b_SensorModel[4],
                                                BYTE b_ConnectionType,
                                                DOUBLE d_RtdResistance[2]);
/**
* Initialize the selected analog-input module for a scan acquisition.
* <b>Before calling this function, you must call i_PCI3200_InitScanModuleAcquisitionSensor function. </b>
* @param[in] h_DeviceHandle            Handle of the board
* @param[in] b_ModuleIndex            Index of the module (0-3)
* @param[in] dw_ConversionFrequency    The frequency of conversion (20Hz, 40Hz, 80Hz or 160Hz)
* @param[in] b_DelayMode            Specify if you want to use a delay (APCI3200_DELAY_NOT_USED or APCI3200_DELAY_MODE1_USED)
* @param[in] dw_DelayTime            The delay time value. It is the time between two scan acquisition (the acquisition time is included). (0-1023)
* @param[in] b_DelayUnit            The unit of dw_DelayTime (APCI3200_UNIT_MS or APCI3200_UNIT_S)
* @param[in] b_AcquisitionMode        Specify if you want to use continuous acquisition or not (APCI3200_SINGLE_SCAN or APCI3200_CONTINUOUS_SCAN)
* @param[in] b_ExtTriggerMode        Specify if you want to use an external trigger (digital input) or not. \n
* <ul>
* <li>APCI3200_DISABLE: do not use external trigger</li>
* <li>APCI3200_FIRST_LOW_EDGE_START_ALL_SCAN: when detecting a falling edge, the acquisition is started</li>
* <li>APCI3200_FIRST_HIGH_EDGE_START_ALL_SCAN: when detecting a rising edge, the acquisition is started</li>
* <li>APCI3200_FIRST_EDGE_START_ALL_SCAN: when detecting a falling or a rising edge, the acquisition is started</li>
* </ul>
* @retval 0 :    No error
* @retval -1 :    The h_DeviceHandle parameter is wrong
* @retval -2 :    The b_ModuleIndex parameter is wrong
* @retval -3 :    The dw_ConversionFrequency parameter is wrong
* @retval -4 :    The b_DelayMode parameter is wrong
* @retval -5 :    The b_DelayUnit parameter is wrong
* @retval -6 :    The dw_DelayTime parameter is wrong
* @retval -7 :    The b_AcquisitionMode parameter is wrong
* @retval -8 :    The b_ExtTriggerMode parameter is wrong
* @retval -9 :    The sensors of the module were not initialized. Call i_PCI3200_InitScanModuleAcquisitionSensor before calling this function
* @retval -10 :    Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCI3200_InitScanModuleAcquisition(HANDLE    h_DeviceHandle,
                                          BYTE b_ModuleIndex,
                                          DWORD dw_ConversionFrequency,
                                          BYTE b_DelayMode,
                                          DWORD dw_DelayTime,
                                          BYTE b_DelayUnit,
                                          BYTE b_AcquisitionMode,
                                          BYTE b_ExtTriggerMode);

/**
* Start a scan acquisition on the selected analog-input module. \n
* Once the acquisition is started, you will receive the values measured by the board with your interrupt routine callback.
* <b>Before calling this function, you must call the function i_PCI3200_InitScanModuleAcquisition. You must also install an interrupt routine.</b>
* @param[in] h_DeviceHandle            Handle of the board
* @param[in] b_ModuleIndex            Index of the module (0-3)
* @retval 0 : No error
* @retval -1 :    The h_DeviceHandle parameter is wrong
* @retval -2 :    The b_ModuleIndex parameter is wrong
* @retval -3 :    No interrupt routine installed. Call the function i_PCI3200_SetBoardIntRoutine before calling this function
* @retval -4 :    The module was not initialized. Call the function i_PCI3200_InitScanModuleAcquisition before calling this function
* @retval -5 :    A scan acquisition is still started on this module. Stop it before trying to start it.
* @retval -6 :    Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCI3200_StartScanModuleAcquisition(HANDLE    h_DeviceHandle,
                                           BYTE b_ModuleIndex);

/**
* Stop a scan acquisition on the selected analog-input module.
* @param[in] h_DeviceHandle            Handle of the board
* @param[in] b_ModuleIndex            Index of the module (0-3)
* @retval 0 : No error
* @retval -1:    The h_DeviceHandle parameter is wrong
* @retval -2:    The b_ModuleIndex parameter is wrong
* @retval -3:    Error while calling IOCTL function (wrong handle ?)
* @retval -4:    The module is not initialized for scan
* @retval -5:    The scan is not started on this module
* @retval -6:    Failed to stop scan on this module    
**/
_INT_ i_PCI3200_StopScanModuleAcquisition(HANDLE    h_DeviceHandle,
                                          BYTE b_ModuleIndex);

/**
 * Reset scan on the selected module
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_ModuleIndex Index of the module (0 to numberOfModules)
 *
 * @retval 0 : No error
 * @retval -1: The h_DeviceHandle parameter is wrong
 * @retval -2: the b_ModuleIndex parameter is wrong
 * @retval -3: the scan is started on this module call i_PCI3200_StopScanModuleAcquisition before.
 * @retval -4: Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCI3200_ReleaseScanModuleAcquisition(HANDLE h_DeviceHandle, UCHAR b_ModuleIndex);

/**
 *        @}
 *        
 *        \addtogroup Temperature xPCI-3200 Temperature function
 *         \ingroup PCI3200
 *        @{ 
 */

/**
* Read the temperature of a connected sensor.
* @param[in] h_DeviceHandle        Handle of the board
* @param[in] b_ModuleIndex        Index of the module (0-3)
* @param[in] b_ChannelIndex        Index of the channel (0-3) If b_ConnectionType is set to differential, then the value must be 0 or 1 (only 2 channels)
* @param[in] b_SensorType        The type of the sensor connected (ADDIDATA_THERMOCOUPLE for TC sensors, ADDIDATA_RTD for RTD sensors)
* @param[in] b_SensorModel        The model of the sensor. \n
* If b_SensorType is ADDIDATA_THERMOCOUPLE : \n
* <ul>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_B</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_E</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_J</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_K</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_N</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_R</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_S</li>
* <li>ADDIDATA_THERMOCOUPLE_TYPE_T</li>
* </ul>
* If b_SensorType is ADDIDATA_RTD : \n
* <ul>
* <li>ADDIDATA_RTD_TYPE_PT</li>
* <li>ADDIDATA_RTD_TYPE_Ni</li>
* </ul>
* @param[in] d_RtdResistance    The resistance R0 of the RTD sensor. Set it to 0 if you do not use RTD sensor. \n
* R0 is the resistance of your sensor for a temperature of 0 Celcius
* @param[in] b_Gain                The gain you want to use.
* <ul>
* <li>- 0: x1</li>
* <li>- 1: x2</li>
* <li>- 2: x4</li>
* <li>- 3: x8</li>
* <li>- 4: x16</li>
* <li>- 5: x32</li>
* <li>- 6: x64</li>
* <li>- 7: x128</li>
* </ul>
* @param[in] b_ConnectionType    The type of the connection that is used (APCI3200_CONNECTION_TYPE_SINGLE for single-ended, and APCI3200_CONNECTION_TYPE_DIFFERENTIAL for differential) \n    
* <b> If b_SensorType is RTD, b_ConnectionType must be set to APCI3200_CONNECTION_TYPE_DIFFERENTIAL </b>
* @param[in] dw_ConversionFrequency    The frequency of conversion (20Hz, 40Hz, 80Hz or 160Hz)
* @param[out] pd_TemperatureValue    A pointer on a value that will be filled with the measured temperature (in Celcius)
* @retval 0 : No error
* @retval -1 :    The h_DeviceHandle parameter is wrong
* @retval -2 :    The b_ModuleIndex parameter is wrong
* @retval -3 :    The b_SensorType parameter is wrong
* @retval -4 :    The b_ConnectionType parameter is wrong
* @retval -5 :    b_SensorType is set to ADDIDATA_RTD, but the connection type is not set to APCI3200_CONNECTION_TYPE_DIFFERENTIAL
* @retval -6 :    The b_ChannelIndex parameter is wrong
* @retval -7 :    The dw_ConversionFrequency parameter is wrong
* @retval -8 :    The b_Gain parameter is wrong
* @retval -9 :    Error while calling IOCTL function (wrong handle ?)
* @retval -10 :    Error while reading the calibration gain value
* @retval -11 :    Error while reading the calibration offset value
* @retval -12 :    Error while reading the calibration values of the channel
* @retval -13 :    Error while reading the CJC value
* @retval -14 :    Error while reading the CJC gain value
* @retval -15 :    Error while reading the CJC offset value
* @retval -16 :    Error while computing the temperature of the CJC
* @retval -17 :    Error while computing the TC-X value of the CJC
* @retval -18 :    Error while computing the temperature
* @retvak -19 :    Sensor model is wrong
* @retval -20 :    Error while computing the RTD temperature
* @retval -30 :    The b_SensorType parameter is wrong
* @retval -100:   Failed to read Eeprom
**/
_INT_ i_PCI3200_Read1Temperature(HANDLE    h_DeviceHandle,
                                 BYTE    b_ModuleIndex,
                                 BYTE    b_ChannelIndex,
                                 BYTE    b_SensorType,
                                 BYTE    b_SensorModel,
                                 DOUBLE d_RtdResistance,
                                 BYTE    b_Gain,
                                 BYTE    b_ConnectionType,
                                 DWORD    dw_ConversionFrequency,
                                 DOUBLE *    pd_TemperatureValue);

/**
 * Read more temperature function allows to read several channels of the board.
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] dw_ConnectionType ConnectionType for each module to read (if module not used, don't have to be set)
 *                           <ul>
 *                               <li> dw_ConnectionType[0] (module 0) \n
 *                                   <ul>
 *                                       <li> APCI3200_CONNECTION_TYPE_SINGLE
 *                                       <li> APCI3200_CONNECTION_TYPE_DIFFERENTIAL
 *                                   </ul>
 *                               </li>
 *                               <li> dw_ConnectionType[1] (module 1) \n
 *                                   <ul>
 *                                       <li> APCI3200_CONNECTION_TYPE_SINGLE
 *                                       <li> APCI3200_CONNECTION_TYPE_DIFFERENTIAL
 *                                   </ul>
 *                               </li>
 *                               <li> dw_ConnectionType[2] (module 2) \n
 *                                   <ul>
 *                                       <li> APCI3200_CONNECTION_TYPE_SINGLE
 *                                       <li> APCI3200_CONNECTION_TYPE_DIFFERENTIAL
 *                                   </ul>
 *                               </li>
 *                               <li> dw_ConnectionType[3] (module 3) \n
 *                                   <ul>
 *                                       <li> APCI3200_CONNECTION_TYPE_SINGLE
 *                                       <li> APCI3200_CONNECTION_TYPE_DIFFERENTIAL
 *                                   </ul>
 *                               </li>
 *                           </ul>
 *
 * @param[in] dw_ChannelMask Channel mask for one module. (Example dw_ChannelMask[0] = 0x4 to read module 0 channel 2)\n
 *                           <ul>
*                                <li> dw_ChannelMask[0] (module 0) \n 
*                                    <ul>
*                                        <li> Bit 0: Channel 0 </li>
*                                        <li> Bit 1: Channel 1 </li>
*                                        <li> Bit 2: Channel 2 (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                        <li> Bit 3: Channel 3 (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                    </ul>
*                                </li>
*                                <li> dw_ChannelMask[1] (module 1) \n 
*                                    <ul>
*                                        <li> Bit 0: Channel 0 </li>
*                                        <li> Bit 1: Channel 1 </li>
*                                        <li> Bit 2: Channel 2 (only if b_ConnectionType[1] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                        <li> Bit 3: Channel 3 (only if b_ConnectionType[1] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                    </ul>
*                                </li>
*                                <li> dw_ChannelMask[2] (module 2) only if APCI3200-16
*                                    <ul>
*                                        <li> Bit 0: Channel 0 </li>
*                                        <li> Bit 1: Channel 1 </li>
*                                        <li> Bit 2: Channel 2 (only if b_ConnectionType[2] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                        <li> Bit 3: Channel 3 (only if b_ConnectionType[2] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                    </ul>
*                                </li>
*                                <li> dw_ChannelMask[3] (module 3) only if APCI3200-16
*                                    <ul>
*                                        <li> Bit 0: Channel 0 </li>
*                                        <li> Bit 1: Channel 1 </li>
*                                        <li> Bit 2: Channel 2 (only if b_ConnectionType[3] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                        <li> Bit 3: Channel 3 (only if b_ConnectionType[3] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                    </ul>
*                                </li>
*                            </ul>
* @param[in] dw_SensorType The type of the sensor connected for each channel to read \n
*                            <ul>
*                                <li> dw_SensorType[0] (module 0) \n
*                                <ul>
*                                    <li> Bit 0..7: Channel 0 configuration
*                                       <ul>
*                                           <li> ADDIDATA_THERMOCOUPLE for TC sensors </li>
*                                           <li> ADDIDATA_RTD for RTD sensors </li>
*                                       </ul>
*                                    </li>
*                                    <li> Bit 8..15: Channel 1 configuration
*                                       <ul>
*                                           <li> ADDIDATA_THERMOCOUPLE for TC sensors </li>
*                                           <li> ADDIDATA_RTD for RTD sensors </li>
*                                       </ul>
*                                    </li>
*                                    <li> Bit 16..23: Channel 2 configuration (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE)
*                                       <ul>
*                                           <li> ADDIDATA_THERMOCOUPLE for TC sensors </li>
*                                           <li> ADDIDATA_RTD for RTD sensors </li>
*                                       </ul>
*                                    </li>
*                                    <li> Bit 24..31: Channel 3 configuration (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE)
*                                       <ul>
*                                           <li> ADDIDATA_THERMOCOUPLE for TC sensors </li>
*                                           <li> ADDIDATA_RTD for RTD sensors </li>
*                                       </ul>
*                                    </li>
*                                </ul>
*                                </li>
*                                <li> dw_SensorType[1] (module 1) ...\n </li>
*                            </ul>
* @param[in] dw_SensorModel The model of the sensor \n
*                           <ul>
*                               <li> dw_SensorModel[0] (module 0)
*                                   <ul>
*                                   <li> Bit 0..7 channel 0 configuration
*                                       <ul>
*                                       <li> If b_SensorType is ADDIDATA_THERMOCOUPLE : \n
*                                           <ul>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_B</li>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_E</li>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_J</li>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_K</li>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_N</li>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_R</li>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_S</li>
*                                               <li>ADDIDATA_THERMOCOUPLE_TYPE_T</li>
*                                           </ul>
*                                       </li>
*                                       <li> If b_SensorType is ADDIDATA_RTD : \n
*                                           <ul>
*                                               <li>ADDIDATA_RTD_TYPE_PT</li>
*                                               <li>ADDIDATA_RTD_TYPE_Ni</li>
*                                           </ul>
*                                       </li>
*                                       </ul>
*                                   </li>
*                                   <li> Bit 8..15 channel 1 configuration...</li>
*                                   <li> Bit 16..23 channel 2 configuration (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE...</li>
*                                   <li> Bit 24..31 channel 3 configuration (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE...</li>
*                               </ul>
*                               <li> dw_SensorModel[1] (module 1)... </li>
*                           </ul>
* @param[in] d_RtdResistance The resistance R0 of the RTD sensor. Set it to 0 if you do not use RTD sensor. \n
* @param[in] dw_Gain Gain for each channel by module
*                           <ul>
*                               <li> dw_Gain[0] (module 0)
*                                   <ul>
*                                       <li> Bit 0..7: Gain channel 0
*                                           <ul>
*                                               <li>- 0: x1</li>
*                                               <li>- 1: x2</li>
*                                               <li>- 2: x4</li>
*                                               <li>- 3: x8</li>
*                                               <li>- 4: x16</li>
*                                               <li>- 5: x32</li>
*                                               <li>- 6: x64</li>
*                                               <li>- 7: x128</li>
*                                           </ul>
*                                       </li>
*                                       <li> Bit 8..15: Gain channel 1
*                                           <ul>
*                                               <li>- 0: x1</li>
*                                               <li>- 1: x2</li>
*                                               <li>- 2: x4</li>
*                                               <li>- 3: x8</li>
*                                               <li>- 4: x16</li>
*                                               <li>- 5: x32</li>
*                                               <li>- 6: x64</li>
*                                               <li>- 7: x128</li>
*                                           </ul>
*                                       </li>
*                                       <li> Bit 16..23: Gain channel 2 (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                       <li> Bit 24..31: Gain channel 3 (only if b_ConnectionType[0] = APCI3200_CONNECTION_TYPE_SINGLE) </li>
*                                   </ul>
*                               </li>
*                               <li> dw_Gain[1] (module 1)... </li>
*                               <li> dw_Gain[2] (module 2)... </li>
*                               <li> dw_Gain[3] (module 3)... </li>
*                           </ul>                                                
*
* @param[in] dw_ConversionFrequency Conversion frequency for each channels
*                                   \li 20: 20 Hz
*                                   \li 40: 40 Hz
*                                   \li 80: 80 Hz
*                                   \li 160: 160 Hz
* @param[in] dw_Trigger Enable / Disable software trigger
*                         \li 0: Software trigger not used
*                         \li 1: Use software trigger (Use i_PCI3200_ReadTemperatureTrigger and i_PCI3200_ReadTemperatureTriggerRelease functions)
* @param[in] dw_Interrupt Enable / Disable interrupt
*                         \li 0: Disable interrupt (data stored in pd_TemperatureValues)
*                         \li 1: Enable interrupt (no data in pd_TemperatureValues)
* @param[in] pd_TemperatureValues Table with all temperature values in module and channel order. This table depends on dw_ChannelMask parameter
*                                   \li Example dw_ChannelMask[0] = 0x4, dw_ChannelMask[2] = 0x2, other dw_ChannelMask = 0\n
*                                   -> pd_TemperatureValues[0] = module 0 channel 2\n
*                                   -> pd_TemperatureValues[1] = module 2 channel 0\n
*                                   -> pd_TemperatureValues[2] = module 2 channel 1\n
*
* @retval -1 :    The h_DeviceHandle parameter is wrong
* @retval -2 :    The dw_ConversionFrequency parameter is wrong
* @retval -3 :    The dw_Interrupt parameter is wrong
* @retval -4 :    The dw_Trigger parameter is wrong
* @retval -5 :    The dw_ConnectionType parameter is wrong
* @retval -6 :    One module is already initialized for read more
* @retval -7 :    One module is already initialized for scan
* @retval -8 :    The dw_ChannelMask parameter is wrong
* @retval -9 :    The dw_SensorType parameter is wrong
* @retval -10:    The dw_Gain parameter is wrong
* @retval -11:    The dw_ChannelMask cannot be equal to 0
* @retval -12:    Failed to get board value
* @retval -13:    Error while calling IOCTL function (wrong handle ?)
* @retval -14:    Interrupt routine is not set
* @retval -100:   Failed to read Eeprom
*
 **/
_INT_ i_PCI3200_ReadMoreTemperature(HANDLE h_DeviceHandle,
                                    DWORD  dw_ConnectionType[4],
                                    DWORD  dw_ChannelMask[4],
                                    DWORD  dw_SensorType[4],
                                    DWORD  dw_SensorModel[4],
                                    DOUBLE d_RtdResistance[8],
                                    DWORD  dw_Gain[4],
                                    DWORD  dw_ConversionFrequency,
                                    DWORD  dw_Interrupt,
                                    DWORD  dw_Trigger,
                                    DOUBLE *pd_TemperatureValues);

/** Allows to run the last read more acquisition without reading all parameters again, only channels values are read again.
 * In i_PCI3200_ReadMoreTemperature function dw_Trigger parameter must be set to 1 in order to use this function.
 *
 *
 * @param[in] h_DeviceHandle        Handle of the board
 * @param[out] pd_TemperatureValue  Temperature values corresponding to dw_ChannelMask from i_PCI3200_ReadMoreTemperature function
 *
 * @retval -1 :    The h_DeviceHandle parameter is wrong
 * @retval -2 :    Error while calling IOCTL function (wrong handle ?)
 * @retval -3 :    Trigger was not activated in i_PCI3200_ReadMoreTemperature function
 * @retval -4 :    An acquisition is already running
 */
_INT_ i_PCI3200_ReadMoreTemperatureTrigger(HANDLE h_DeviceHandle,
                                       DOUBLE *pd_TemperatureValue);
									   
/** Release the acquisition started by i_PCI3200_ReadMoreTemperature function with trigger selected
 *
 * In i_PCI3200_ReadMoreTemperature function dw_Trigger parameter must be set to 1 in order to use this function.
 *
 * @param[in] h_DeviceHandle        Handle of the board
 * @retval -1 :    The h_DeviceHandle parameter is wrong
 * @retval -2 :    Error while calling IOCTL function (wrong handle ?)
 * @retval -3 :    Trigger was not activated in i_PCI3200_ReadMoreTemperature function
 */
_INT_ i_PCI3200_ReadMoreTemperatureTriggerRelease(HANDLE h_DeviceHandle);
                                    

/**
 *        @}
 *        
 *        \addtogroup Diagnostic xPCI-3200 Diagnostic functions
 *         \ingroup PCI3200
 *        @{ 
 */

/**
* Check the open input status of a sensor. This function is available for TC and RTD sensors.
* @param[in] h_DeviceHandle        Handle of the board
* @param[in] b_ModuleIndex        Index of the module (0-3)
* @param[in] b_ChannelIndex        Index of the channel (0-3) If b_ConnectionType is set to differential, then the value must be 0 or 1 (only 2 channels)
* @param[in] b_ConnectionType    The type of the connection that is used (APCI3200_CONNECTION_TYPE_SINGLE for single-ended, and APCI3200_CONNECTION_TYPE_DIFFERENTIAL for differential)
* @param[out] pb_Diagnostic        A pointer to a byte that will be filled with the diagnostic status. 0: no open input, 1: open input detected
* @retval 0 : No error
* @retval -1 :    The h_DeviceHandle parameter is wrong
* @retval -2 :    The b_ModuleIndex parameter is wrong
* @retval -3 :    The b_ChannelIndex parameter is wrong
* @retval -4 :    The b_ConnectionType parameter is wrong
* @retval -5 :    Error while calling IOCTL function (wrong handle ?)
* @retval -6 :    Error while reading the calibration gain value
* @retval -7 :    Error while reading the calibration offset value
* @retval -8 :    Error while reading the calibration values of the channel
**/
_INT_ i_PCI3200_GetOpenInputDiagnostic(HANDLE    h_DeviceHandle,
                                       BYTE b_ModuleIndex,
                                       BYTE b_ChannelIndex,
                                       BYTE b_ConnectionType,
                                       PBYTE pb_Diagnostic);

/**
* Check the short circuit status of a sensor. <b>This function is only available for RTD sensors.</b>
* @param[in] h_DeviceHandle        Handle of the board
* @param[in] b_ModuleIndex        Index of the module (0-3)
* @param[in] b_ChannelIndex        Index of the channel (0-1)
* @param[out] pb_Diagnostic        A pointer to a byte that will be filled with the diagnostic status. 0: no short-circuit, 1: short-circuit detected
* @retval 0 : No error
* @retval -1 :    The h_DeviceHandle parameter is wrong
* @retval -2 :    The b_ModuleIndex parameter is wrong
* @retval -3 :    The b_ChannelIndex parameter is wrong
* @retval -4 :    Error while calling IOCTL function (wrong handle ?)
* @retval -5 :    Error while reading the calibration gain value
* @retval -6 :    Error while reading the calibration offset value
* @retval -7 :    Error while reading the calibration values of the channel
**/
_INT_ i_PCI3200_GetShortCircuitDiagnostic(HANDLE    h_DeviceHandle,
                                          BYTE b_ModuleIndex,
                                          BYTE b_ChannelIndex,
                                          PBYTE pb_Diagnostic);

/**
 *        @}
 *    @}
 */
#ifdef __cplusplus
    }
#endif
