using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3200_NET
{
    public class PCI3200
    {
        /// <summary>
        /// Store the index of the board
        /// </summary>
        private int index;

        /// <summary>
        /// Store an handle to the board. Will be used for interrupt routine for example
        /// </summary>
        private IntPtr handle;

        /// <summary>
        /// An object that will contain the analog input event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private AnalogInputEventArgs analogInputEventArgs;

        /// <summary>
        /// An object that will contain the end of scan event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private EndOfScanEventArgs endOfSscanEventArgs;

        /// <summary>
        /// Handler to the interrupt function of the board
        /// The function pointed by this delegate will be called each time the board interrupts
        /// </summary>
        private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

        #region "Event"

        /// <summary>
        /// The analog input event
        /// </summary>
        public event AnalogInputEventHandler AnalogInputEventReceived;

        /// <summary>
        /// Delegate to provide when an interrupt on the analog input occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void AnalogInputEventHandler(object source, AnalogInputEventArgs args);

        /// <summary>
        /// The end of scan event
        /// </summary>
        public event EndOfSscanEventHandler EndOfScanEventReceived;

        /// <summary>
        /// Delegate to provide when an end of scan interrupt occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void EndOfSscanEventHandler(object source, EndOfScanEventArgs args);

        /// <summary>
        /// The pld fifo overflow event
        /// </summary>
        public event PldFifoOverflowEventHandler PldFifoOverflowEventReceived;

        /// <summary>
        /// Delegate to provide when a PLD fifo overflow occurs
        /// </summary>
        /// <param name="source"></param>
        public delegate void PldFifoOverflowEventHandler(object source);

        #endregion

        #region "Constructor/Destructor"

        /// <summary>
        /// Constructor of the board
        /// Cannot be called; initialize the internal variables
        /// </summary>
        private PCI3200()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args (once by board)
            this.analogInputEventArgs = new AnalogInputEventArgs();
            this.endOfSscanEventArgs = new EndOfScanEventArgs();

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);
        }

        /// <summary>
        /// Destructor of the board
        /// Free all the ressources
        /// </summary>
        ~PCI3200()
        {
            DllImport.i_PCI3200_CloseBoard(handle);
            this.index = -1;
            this.handle = IntPtr.Zero;
        }

        #endregion

        #region "Utils"

        /// <summary>
        /// Enables to get an instance of a PCI3200 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCI3200 on success</returns>
        public static PCI3200 OpenBoard(int index)
        {
            PCI3200 boardInstance = null;
            int nbBoard = 0;
            IntPtr handle = IntPtr.Zero;

            // Get the number of boards
            if ((nbBoard = GetBoardNumber()) == -1)
            {
                Console.WriteLine("GetBoardNumber failed");
                return null;
            }

            // Check that the index is in the range
            if (index < 0 || index > nbBoard)
            {
                Console.WriteLine("index not valid");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCI3200_OpenBoardViaIndex((byte)index, ref handle) != 0)
            {
                Console.WriteLine("i_PCI3200_OpenBoardViaIndex failed");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCI3200();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Enables to get the number of PCI3200 boards plugged in the computer
        /// </summary>
        /// <returns>The number of board</returns>
        public static int GetBoardNumber()
        {
            int ret = 0;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCI3200_GetNumberOfBoards(ref nbBoard);

            // Error case
            if (ret != 0) { return -1; }

             // Success
            else { return (int)nbBoard; }
        }

        /// <summary>
        /// Enables to signal to all the listeners that a PLD fifo overflow occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnPldFifoOverflowEventReceived()
        {
            if (this.PldFifoOverflowEventReceived != null)
            {
                this.PldFifoOverflowEventReceived(this);
            }
        }

        #endregion

        #region "Interrupt"

        /// <summary>
        /// Interrupt function - Called each time the board generates an interrupt
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="overflow"></param>
        /// <param name="source"></param>
        /// <param name="mask"></param>
        /// <param name="args"></param>
        /// <param name="argsSize"></param>
        private void InterruptFunction(IntPtr handle, byte overflow,uint source,uint mask,IntPtr args, uint argsSize )
        {
            // PLD fifo overflow ?
            if ((source & 0x10) == 0x10)
            {
                this.OnPldFifoOverflowEventReceived();
            }
            // end of scan ?
            else if ((mask & 1) == 1)
            {
                byte[] data = new byte[argsSize * sizeof(double)];

                this.endOfSscanEventArgs.InterruptMask = mask;
                this.endOfSscanEventArgs.InterruptSource = source;
                this.endOfSscanEventArgs.FifoOverflow = overflow;
                this.endOfSscanEventArgs.ArgsSize = argsSize;

                // Convert the args
                Marshal.Copy(args, data, 0, (int)argsSize * sizeof(double));
                for (int i = 0; i < argsSize; i++)
                {
                    this.endOfSscanEventArgs.Args[i] = BitConverter.ToDouble(data, i * sizeof(double));
                }

                // Signal the event
                this.OnEndOfScanEventReceived(this.endOfSscanEventArgs);
            }
        }

        /// <summary>
        /// Set the board interrupt routine
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetBoardIntRoutine()
        {
            return (int)DllImport.i_PCI3200_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        /// <returns></returns>
        public int ResetBoardIntRoutine()
        {
            return (int)DllImport.i_PCI3200_ResetBoardIntRoutine(this.handle);
        }

        #endregion

        #region "Digital outputs"

        /// <summary>
        /// Set the digital memory of the outputs to on
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetDigitalOutputMemoryOn()
        {
            return (int)DllImport.i_PCI3200_SetDigitalOutputMemoryOn(this.handle);
        }

        /// <summary>
        /// Set the digital memory of the outputs to off
        /// </summary>
        /// <returns></returns>
        public int SetDigitalOutputMemoryOff()
        {
            return (int)DllImport.i_PCI3200_SetDigitalOutputMemoryOff(this.handle);
        }

        /// <summary>
        /// Enables to set one channel to on (high state)
        /// </summary>
        /// <param name="channel">The index of the channel (start at 0)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set1DigitalOutputOn(byte channel)
        {
            return (int)DllImport.i_PCI3200_Set1DigitalOutputOn(this.handle, channel);
        }

        /// <summary>
        /// Enables to set one channel to off (low state)
        /// </summary>
        /// <param name="channel">The index of the channel (start at 0)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set1DigitalOutputOff(byte channel)
        {
            return (int)DllImport.i_PCI3200_Set1DigitalOutputOff(this.handle, channel);
        }

        /// <summary>
        /// Enables to set several digital outputs to on (high state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 7)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set3DigitalOutputsOn(byte value)
        {
            return (int)DllImport.i_PCI3200_Set3DigitalOutputsOn(this.handle, value);
        }

        /// <summary>
        /// Enables to set several digital outputs to off (low state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 7)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set3DigitalOutputsOff(byte value)
        {
            return (int)DllImport.i_PCI3200_Set3DigitalOutputsOff(this.handle, value);
        }

        /// Read 1 digital output status
        /// </summary>
        /// <param name="channel">Channel to be read (0 to 31)</param>
        /// <param name="outputStatus">Status of the digital outputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Get1DigitalOutputsStatus(byte channel, ref byte outputStatus)
        {
            return (int) DllImport.i_PCI3200_Get1DigitalOutputStatus(this.handle, channel, ref outputStatus);
        }

        /// <summary>
        /// Read 3 digital outputs status
        /// </summary>
        /// <param name="value">Status of the digital outputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Get3DigitalOutputsStatus(ref byte outputStatus)
        {
            return (int)DllImport.i_PCI3200_Get3DigitalOutputsStatus(this.handle, ref outputStatus);
        }

        #endregion

        #region "Digital inputs"

        /// <summary>
        /// Enables to read one digital
        /// </summary>
        /// <param name="channel">Channel to read (0-8)</param>
        /// <param name="inputValue">A byte that will be filled with the current status of the input (0,1)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read1DigitalInput(byte channel, ref byte inputValue)
        {
            // Call the function
            return DllImport.i_PCI3200_Read1DigitalInput(this.handle, channel, ref inputValue);
        }

        /// <summary>
        /// Enables to read 4 digital inputs (per port)
        /// </summary>
        /// <param name="inputValues">A byte that will be filled with the current status of the inputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read4DigitalInputs(ref byte inputValues)
        {
            // Call the function
            return DllImport.i_PCI3200_Read4DigitalInputs(this.handle, ref inputValues);
        }

        #endregion

        #region "Analog inputs"

        /// <summary>
        /// Enables to signal to all the listeners that an event on the analog inputs occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnAnalogInputEventReceived(AnalogInputEventArgs eventArgs)
        {
            if (this.AnalogInputEventReceived != null)
            {
                this.AnalogInputEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Enables to signal to all the listeners that an end of scan interrupt occurs
        /// </summary>
        /// <param name="eventArgs"></param>
        protected virtual void OnEndOfScanEventReceived(EndOfScanEventArgs eventArgs)
        {
            if (this.EndOfScanEventReceived != null)
            {
                this.EndOfScanEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Read the temperature of one channel
        /// </summary>
        /// <param name="moduleIndex">Index of the module where the channel is (0-3)</param>
        /// <param name="channelIndex">Index of the channel in the module (0-3)</param>
        /// <param name="sensorType">Type of the sensor. (0: TC, 1: RTD)</param>
        /// <param name="sensorModel">Model of the sensor.
        /// If sensor type is TC:
        /// - 0: Type B
        /// - 1: Type E
        /// - 2: Type J
        /// - 3: Type K
        /// - 4: Type N
        /// - 5: Type R
        /// - 6: Type S
        /// - 7: Type T
        /// If sensor type is RTD: 
        /// - 0: PT
        /// - 1: Ni</param>
        /// <param name="rtdResistance">If the sensor type is RTD, specify the zero resistance of the sensor</param>
        /// <param name="gain">Selected gain
        /// - 0: x1
        /// - 1: x2
        /// - 2: x4
        /// - 3: x8
        /// - 4: x16
        /// - 5: x32
        /// - 6: x64
        /// - 7: x128</param>
        /// <param name="connectionType">Type of the connection
        /// - 0: Single ended
        /// - 1: Differential</param>
        /// <param name="conversionFrequency">The frequency of conversion (20Hz, 40Hz, 80Hz or 120Hz)</param>
        /// <param name="temperature">Value given by reference. Will contain the temperature value in Celcius</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read1Temperature(byte moduleIndex,
            byte channelIndex,
            byte sensorType,
            byte sensorModel,
            double rtdResistance,
            byte gain,
            byte connectionType,
            uint conversionFrequency,
            ref double temperature)
        {
             return DllImport.i_PCI3200_Read1Temperature(this.handle,moduleIndex,channelIndex,sensorType,sensorModel,rtdResistance,gain,connectionType,conversionFrequency,ref  temperature);
        }

        /// <summary>
        /// Initialize a module.
        /// This function must be called before the InitScanModuleAcquisition function.
        /// </summary>
        /// <param name="moduleIndex">Index of the module (0-3)</param>
        /// <param name="sensorType">Type of the sensor. (0: TC, 1: RTD)</param>
        /// <param name="sensorModel">Array that contains the model of the sensors.
        /// If sensor type is TC:
        /// - 0: Type B
        /// - 1: Type E
        /// - 2: Type J
        /// - 3: Type K
        /// - 4: Type N
        /// - 5: Type R
        /// - 6: Type S
        /// - 7: Type T
        /// If sensor type is RTD: 
        /// - 0: PT
        /// - 1: Ni</param>
        /// <param name="connectionType">Type of the connection
        /// - 0: Single ended
        /// - 1: Differential</param>
        /// <param name="rtdResistance">If you choose the RTD type, specify the R0 resistance of your sensor (ex: 100 for PT100, 1000 for PT1000). One value per RTD sensor (so 2 values)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitScanModuleAcquisitionSensor(byte moduleIndex,
            byte sensorType,
            byte[] sensorModel,
            byte connectionType,
            double[] rtdResistance)
        {
            return DllImport.i_PCI3200_InitScanModuleAcquisitionSensor(this.handle, moduleIndex, sensorType, sensorModel, connectionType, rtdResistance);
        }

        /// <summary>
        /// Initialize a scan acquisition.
        /// The scan acquisition enables to measure the values of all the channels of a module.
        /// When the measure is done, you will receive an interrupt containing the measured temperature.
        /// Before calling this function, you have to call InitScanModuleAcquisitionSensor
        /// </summary>
        /// <param name="moduleIndex">Index of the module (0-3)</param>
        /// <param name="conversionFrequency">The frequency of acquisition (20Hz, 40Hz, 80Hz or 160Hz)</param>
        /// <param name="delayMode">0: do not use delay, 1: use delay between two acquisitions</param>
        /// <param name="delay">The value of the delay (0-1023)</param>
        /// <param name="delayUnit">The unit of the delay.
        /// - 2: milli second
        /// - 3: second</param>
        /// <param name="mode">The acquisition mode.
        /// - 0: only one scan (measure only one time all the channels)
        /// - 2: continuous acquisition</param>
        /// <param name="extTriggerMode">The trigger mode.
        /// - 0: disabled
        /// - 1: low edge start the acquisition
        /// - 2: high edge start the acquisition
        /// - 3: both edges (low and high) start the acquisition</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitScanModuleAcquisition(byte moduleIndex,
            uint conversionFrequency,
            byte delayMode,
            uint delayTime,
            byte delayUnit,
            byte acquisitionMode,
            byte extTriggerMode)
        {
            return DllImport.i_PCI3200_InitScanModuleAcquisition(this.handle, moduleIndex, conversionFrequency, delayMode, delayTime, delayUnit, acquisitionMode, extTriggerMode);
        }

        /// <summary>
        /// Start the scan acquisition on a module
        /// Before calling this function, you must call InitScanModuleAcquisition
        /// </summary>
        /// <param name="moduleIndex">Index of the module (0-3)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int StartScanModuleAcquisition(byte moduleIndex)
        {
            return DllImport.i_PCI3200_StartScanModuleAcquisition(this.handle, moduleIndex);
        }

        /// <summary>
        /// Stop the scan acquisition on a module
        /// </summary>
        /// <param name="moduleIndex">Index of the module (0-3)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int StopScanModuleAcquisition(byte moduleIndex)
        {
            return DllImport.i_PCI3200_StopScanModuleAcquisition(this.handle, moduleIndex);
        }

        #endregion

        #region "Diagnostic"

        /// <summary>
        /// Get the open input diagnostic of a channel
        /// </summary>
        /// <param name="moduleIndex">Index of the module where the channel is (0-3)</param>
        /// <param name="channelIndex">Index of the channel in the module (0-3)</param>
        /// <param name="connectionType">Type of the connection
        /// - 0: Single ended
        /// - 1: Differential</param>
        /// <param name="diagnostic">Value given by reference. Will contain the diagnostic status. 0: no open input, 1: open input detected</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int GetOpenInputDiagnostic(byte moduleIndex,
            byte channelIndex,
            byte connectionType,
            ref byte diagnostic)
        {
            return DllImport.i_PCI3200_GetOpenInputDiagnostic(this.handle, moduleIndex, channelIndex, connectionType, ref diagnostic);
        }

        /// <summary>
        /// Get the short-circuit diagnostic of a channel
        /// </summary>
        /// <param name="moduleIndex">Index of the module where the channel is (0-3)</param>
        /// <param name="channelIndex">Index of the channel in the module (0-3)</param>
        /// <param name="diagnostic">Value given by reference. Will contain the diagnostic status. 0: no short-circuit, 1: short-circuit detected</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int GetShortCircuitDiagnostic(byte moduleIndex,
            byte channelIndex,
            ref byte diagnostic)
        {
            return DllImport.i_PCI3200_GetShortCircuitDiagnostic(this.handle, moduleIndex, channelIndex, ref diagnostic);
        }

        #endregion
    }
}
