using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Read1Temperature
{
    public partial class Read1Temperature : Form
    {
        private String[] tCSensorModel = { "B", "E", "J", "K", "N", "R", "S", "T" };
        private String[] rTDSensorModel = { "PT", "Ni" };

        private String[] singleEndedChannels = { "0", "1", "2", "3" };
        private String[] differentialChannels = { "0", "1" };

        /// <summary>
        /// Handle to the first board
        /// </summary>
        private PCI3200_NET.PCI3200 board;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public Read1Temperature()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Read1Temperature_Load(object sender, EventArgs e)
        {
            // Open the first board of the system
            this.board = PCI3200_NET.PCI3200.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Data source for channel selection
            this.channelIndexComboBox.DataSource = singleEndedChannels;

            // Initialize the combo boxes
            this.moduleIndexComboBox.SelectedIndex = 0;
            this.sensorTypeComboBox.SelectedIndex = 0;
            this.gainComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Selected index in the combo box changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sensorTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Set the correct data source
            if (this.sensorTypeComboBox.SelectedIndex == 0)
            {
                // TC
                this.sensorModelComboBox.DataSource = this.tCSensorModel;
                this.rtdResistanceLabel.Visible = false;
                this.rtdResistanceTextBox.Visible = false;
                this.connectionLabel.Visible = true;
                this.connectionComboBox.Visible = true;

                this.connectionComboBox.SelectedIndex = 0;
            }
            else if (this.sensorTypeComboBox.SelectedIndex == 1)
            {
                // RTD
                this.sensorModelComboBox.DataSource = this.rTDSensorModel;
                this.rtdResistanceLabel.Visible = true;
                this.rtdResistanceTextBox.Visible = true;
                this.connectionLabel.Visible = false;
                this.connectionComboBox.Visible = false;

                this.connectionComboBox.SelectedIndex = 1;
            }
        }

        /// <summary>
        /// Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            // Create the parameters for the worker
            ReadWorkerParams workerParams = new ReadWorkerParams();
            workerParams.moduleIndex = (byte)this.moduleIndexComboBox.SelectedIndex;
            workerParams.channelIndex = (byte)this.channelIndexComboBox.SelectedIndex;
            workerParams.sensorType = (byte)this.sensorTypeComboBox.SelectedIndex;
            workerParams.sensorModel = (byte)this.sensorModelComboBox.SelectedIndex;
            workerParams.gain = (byte)this.gainComboBox.SelectedIndex;
            workerParams.conversionFrequency = 20; // 20 Hz
            workerParams.board = this.board;

            if (this.sensorTypeComboBox.SelectedIndex == 0)
            {
                // TC
                workerParams.connectionType = (byte)this.connectionComboBox.SelectedIndex;
                
            }
            else if (this.sensorTypeComboBox.SelectedIndex == 1)
            {
                // RTD
                // Always differential
                workerParams.connectionType = 1;
                // Get the resistance value
                try
                {
                    workerParams.resistance = Convert.ToDouble(this.rtdResistanceTextBox.Text);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Wrong resistance value : " + ex.Message);
                    return;
                }
            }
            else
            {
                return;
            }


            this.readWorker.RunWorkerAsync((object)workerParams);
            this.startButton.Enabled = false;
            this.stopButton.Enabled = true;
        }

        /// <summary>
        /// Start to work
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;
            ReadWorkerParams workerParams = (ReadWorkerParams)e.Argument;

            for (; ; )
            {
                // Must quit ?
                if (worker.CancellationPending)
                {
                    return;
                }
                else
                {
                    double value = 0.0;
                    int returnValue = 0;

                    System.Threading.Thread.Sleep(100);

                    if ( ( returnValue = board.Read1Temperature(workerParams.moduleIndex,
                        workerParams.channelIndex,
                        workerParams.sensorType,
                        workerParams.sensorModel,
                        workerParams.resistance,
                        workerParams.gain,
                        workerParams.connectionType,
                        workerParams.conversionFrequency,
                        ref value)) != 0)
                    {
                        worker.ReportProgress(1, (object)returnValue);
                        return;
                    }
                    else
                    {
                        worker.ReportProgress(2, (object)value);
                    }
                }
            }
        }

        /// <summary>
        /// The worker changed its status
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (e.ProgressPercentage == 1)
            {
                MessageBox.Show("Error while reading temperature : " + (int)e.UserState);
                this.startButton.Enabled = true;
                this.stopButton.Enabled = false;
            }
            else if (e.ProgressPercentage == 2)
            {
                double value = (double) e.UserState;
                this.valueLabel.Text = String.Format("{0:0.00}",value) + " C";
            }
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            this.readWorker.CancelAsync();
            this.startButton.Enabled = true;
            this.stopButton.Enabled = false;
        }

        /// <summary>
        /// Change the connection type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Change the possible channel selection
            if (this.connectionComboBox.SelectedIndex == 0)
            {
                this.channelIndexComboBox.DataSource = singleEndedChannels;
            }
            else if (this.connectionComboBox.SelectedIndex == 1)
            {
                this.channelIndexComboBox.DataSource = differentialChannels;
            }
        }
    }
}