using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Diagnostic
{
    public partial class Diagnostic : Form
    {
        private String[] tCSensorModel = { "B", "E", "J", "K", "N", "R", "S", "T" };
        private String[] rTDSensorModel = { "PT", "Ni" };

        private String[] singleEndedChannels = { "0", "1", "2", "3" };
        private String[] differentialChannels = { "0", "1" };

        /// <summary>
        /// Handle to the first board
        /// </summary>
        private PCI3200_NET.PCI3200 board;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public Diagnostic()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Diagnostic_Load(object sender, EventArgs e)
        {
            // Open the first board of the system
            this.board = PCI3200_NET.PCI3200.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Data source for channel selection
            this.channelIndexComboBox.DataSource = singleEndedChannels;

            // Initialize the combo boxes
            this.moduleIndexComboBox.SelectedIndex = 0;
            this.connectionComboBox.SelectedIndex = 0;
            this.sensorTypeValue.SelectedIndex = 0;
        }

        
        /// <summary>
        /// Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            // Create the parameters for the worker
            DiagnosticWorkerParams workerParams = new DiagnosticWorkerParams();
            workerParams.moduleIndex = (byte)this.moduleIndexComboBox.SelectedIndex;
            workerParams.channelIndex = (byte)this.channelIndexComboBox.SelectedIndex;
            workerParams.connectionType = (byte)this.connectionComboBox.SelectedIndex;
            workerParams.sensorType = (byte)this.sensorTypeValue.SelectedIndex;
            workerParams.board = this.board;


            this.readWorker.RunWorkerAsync((object)workerParams);
            this.startButton.Enabled = false;
            this.stopButton.Enabled = true;
        }

        /// <summary>
        /// Start to work
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;
            DiagnosticWorkerParams workerParams = (DiagnosticWorkerParams)e.Argument;

            for (; ; )
            {
                // Must quit ?
                if (worker.CancellationPending)
                {
                    return;
                }
                else
                {
                    byte diagnostic = 0;
                    int returnValue = 0;

                    System.Threading.Thread.Sleep(100);

                    // Get the open input diagnostic
                    if ( (returnValue = board.GetOpenInputDiagnostic(workerParams.moduleIndex,
                        workerParams.channelIndex,
                        workerParams.connectionType,
                        ref diagnostic) ) != 0 )
                    {
                        worker.ReportProgress(1, (object)returnValue);
                        return;
                    }
                    else
                    {
                        worker.ReportProgress(2, (object)diagnostic);
                    }

                    // If RTD, we check the short-circuit
                    if (workerParams.sensorType == 1)
                    {
                        byte shortCircuitDiagnostic = 0;
                        if ((returnValue = board.GetShortCircuitDiagnostic(workerParams.moduleIndex,
                            workerParams.channelIndex,
                            ref shortCircuitDiagnostic)) != 0)
                        {
                            worker.ReportProgress(3, (object)returnValue);
                            return;
                        }
                        else
                        {
                            worker.ReportProgress(4, (object)shortCircuitDiagnostic);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// The worker changed its status
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (this.stopButton.Enabled == false)
            {
                return;
            }

            if (e.ProgressPercentage == 1)
            {
                MessageBox.Show("Error while getting open-input diagnostic : " + (int)e.UserState);
                this.startButton.Enabled = true;
                this.stopButton.Enabled = false;
            }
            else if (e.ProgressPercentage == 2)
            {
                byte diagnostic = (byte)e.UserState;
                if (diagnostic == 0)
                {
                    this.valueLabel.Text = "No";
                    this.valueLabel.ForeColor = Color.Green;
                }
                else
                {
                    this.valueLabel.Text = "Yes";
                    this.valueLabel.ForeColor = Color.Red;
                }
            }
            else if (e.ProgressPercentage == 3)
            {
                MessageBox.Show("Error while getting short-circuit diagnostic : " + (int)e.UserState);
                this.startButton.Enabled = true;
                this.stopButton.Enabled = false;
            }
            else if (e.ProgressPercentage == 4)
            {
                byte diagnostic = (byte)e.UserState;
                if (diagnostic == 0)
                {
                    this.shortCircuitValueLabel.Text = "No";
                    this.shortCircuitValueLabel.ForeColor = Color.Green;
                }
                else
                {
                    this.shortCircuitValueLabel.Text = "Yes";
                    this.shortCircuitValueLabel.ForeColor = Color.Red;
                }
            }
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            this.readWorker.CancelAsync();

            this.startButton.Enabled = true;
            this.stopButton.Enabled = false;

            this.valueLabel.Text = "-";
            this.valueLabel.ForeColor = Color.Black;

            this.shortCircuitValueLabel.Text = "-";
            this.shortCircuitValueLabel.ForeColor = Color.Black;
        }

        /// <summary>
        /// Change the connection type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Change the possible channel selection
            if (this.connectionComboBox.SelectedIndex == 0)
            {
                this.channelIndexComboBox.DataSource = singleEndedChannels;
            }
            else if (this.connectionComboBox.SelectedIndex == 1)
            {
                this.channelIndexComboBox.DataSource = differentialChannels;
            }
        }

        /// <summary>
        /// Change the sensor type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sensorTypeValue_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Change the possible channel selection
            if (this.sensorTypeValue.SelectedIndex == 0)
            {
                // TC
                this.channelIndexComboBox.DataSource = singleEndedChannels;
                this.connectionComboBox.SelectedIndex = 0;
                this.connectionComboBox.Visible = true;
                this.connectionLabel.Visible = true;
            }
            else
            {
                // RTD
                this.channelIndexComboBox.DataSource = differentialChannels;
                this.connectionComboBox.SelectedIndex = 1;
                this.connectionComboBox.Visible = false;
                this.connectionLabel.Visible = false;
            }
        }
    }
}